// 01.11.07 - add repear offset (linear in maya) in pre and post behavior
//				- add pre behavior

@warnings 0
@define INTEGER 4
@define DOUBLE 8
@define POSTB_DISSOLVE 1
@define POSTB_FIRSTFRAME 2
@define POSTB_LASTFRAME 3
@define POSTB_LOOP 4
@define POSTB_LINEAR 5
@define PREB_DISSOLVE 1
@define PREB_FIRSTFRAME 2
@define PREB_LASTFRAME 3
@define PREB_LOOP 4
@define PREB_LINEAR 5
           
@script motion

frameOffset; defFile; motDir; applyLoadLimit; loadLimit; f;
duration; objType; diag; preb; postb; speed;
firstFramePos; firstFrameRot; firstFrameScl;
isAnim_focalLength; isAnim_shutterAngle; isAnim_focalDistance; isAnim_fStop;
mySelf; mySelfOA;
cstr_type;
posx;posy;posz;roth;rotp;rotb;sclx;scly;sclz;
startFrame; endFrame;
create: obj
{
//	diag = Diagnostic();
//	diag.showonwrite = true;
	mySelfOA = obj;
	mySelf = obj.name;
	setdesc("Maya2LW2 Motion");
	contentDir = getdir("Content");
	appliedBySCGN = integer(globalrecall("M2L_SCGN_APPLY",0));
	posx = posy = posz = roth = rotp = rotb = sclx = scly = sclz = 1;
	defFile = recall("defFile",(string(contentDir,"\\*.m2l")));
	(d1,d2) = split(defFile);
	defFile = string(d1,d2,"*.m2l");
	if(defFile.contains(contentDir) == false)
		defFile = string(contentDir,"\\*.m2l");

	speed = number(recall("speed",1));
	preb = integer(recall("preb",PREB_FIRSTFRAME));
	postb = integer(recall("postb",POSTB_LASTFRAME));
	frameOffset = integer(recall("frameOffset",1));

	if(appliedBySCGN == true)
	{
		defFile = globalrecall("M2L_SCGN_FN",defFile);
		speed = number(globalrecall("M2L_SCGN_SPEED",speed));
		//preb = integer(globalrecall("M2L_SCGN_PREB",preb));
		postb = integer(globalrecall("M2L_SCGN_POSTB",postb));
		
		frameOffset = integer(globalrecall("M2L_SCGN_TGTSTART",frameOffset));
	}

}

queryAttributes
{
	if(m2lsc$queryObjectName == mySelf)
		globalstore("M2L_queryFilename",defFile);
}
updateAttributes
{
	if(m2lsc$queryObjectName == mySelf)
	{
		offset = exposedTargetStart;
		postb = exposedPostBehavior;
		defFile = exposedM2LFilename;
		speed = exposedSpeed;
	}
}
filterMotion: oarray, type, time
{
	farray = <0,0,0>;
	
	if(type == 1)
	{
		s = mySelfOA.getPosition(time);
		if(posx == true)
			farray.x = oarray.x;
		else
			farray.x = s.x;
		if(posy == true)
			farray.y = oarray.y;
		else
			farray.y = s.y;
		if(posz == true)
			farray.z = oarray.z;
		else
			farray.z = s.z;
	}
	
	if(type == 2)
	{
		s = mySelfOA.getRotation(time);
		if(roth == true)
			farray.x = oarray.x;
		else
			farray.x = s.x;
		if(rotp == true)
			farray.y = oarray.y;
		else
			farray.y = s.y;
		if(rotb == true)
			farray.z = oarray.z;
		else
			farray.z = s.z;
	}
	
	if(type == 3)
	{
		s = mySelfOA.getScaling(time);
		if(sclx == true)
			farray.x = oarray.x;
		else
			farray.x = s.x;
		if(scly == true)
			farray.y = oarray.y;
		else
			farray.z = s.y;
		if(sclz == true)
			farray.z = oarray.z;
		else
			farray.z = s.z;
	}
	return(farray);
	
}
process: ma, frame, time
{
	
	if(m2lsc$updateSignal == true)
		updateAttributes();
	if(m2lsc$querySignal == true)
		queryAttributes();
	if(f)
	{
		myPos = <0,0,0>;
		myRot = <0,0,0>;
		myScl = <1,1,1>;
		if(f.offset() == -1)
			f.reopen("rb");
		myFrame = ((time * Scene().fps) - frameOffset) / speed;
		
		if(myFrame < startFrame - frameOffset) // pre behavior
		{
			if(preb == PREB_FIRSTFRAME)
			{
				if(posx != false || posy != false || posz != false)
					ma.set(POSITION,filterMotion(firstFramePos,1,time));
				if(roth != false || rotp != false || rotb != false)
					ma.set(ROTATION,filterMotion(firstFrameRot,2,time));
				if(sclx != false || scly != false || sclz != false)
					ma.set(SCALING,filterMotion(firstFrameScl,3,time));
				return;
			}
			if(preb == PREB_LASTFRAME) // last frame
			{
				myFrame = endFrame - frameOffset;
				
			}
			if(preb == PREB_LOOP)
			{
				while(myFrame >= duration)
					myFrame = myFrame - startFrame;
			}
			if(preb == PREB_LINEAR)
			{
				myPos1 = findDataAtFrame(0,0);
				myRot1 = findDataAtFrame(0,3);
				myScl1 = findDataAtFrame(0,6);
				myPos2 = findDataAtFrame(1,0);
				myRot2 = findDataAtFrame(1,3);
				myScl2 = findDataAtFrame(1,6);
				
				vecPos = myPos2 - myPos1;
				vecRot = myRot2 - myRot1;
				vecScl = myScl2 - myScl1;
				info(vecPos);
				// how many frames behind start?
				
				behindFrame =  (startFrame - frameOffset) - myFrame;
				
				ma.set(POSITION,filterMotion(behindFrame * vecPos,1,time));
				ma.set(ROTATION,filterMotion(behindFrame * vecRot,1,time));
				ma.set(SCALING,filterMotion(behindFrame * vecScl,1,time));
			}
			
		}
		
		if(myFrame > endFrame - frameOffset) // post behavior
		{
			if(postb == POSTB_FIRSTFRAME) // first frame
			{
				if(posx != false || posy != false || posz != false)
					ma.set(POSITION,filterMotion(firstFramePos,1,time));
				if(roth != false || rotp != false || rotb != false)
					ma.set(ROTATION,filterMotion(firstFrameRot,2,time));
				if(sclx != false || scly != false || sclz != false)
					ma.set(SCALING,filterMotion(firstFrameScl,3,time));
				return;
			}
			if(postb == POSTB_LASTFRAME) // last frame
			{
				myFrame = endFrame - frameOffset;
				
			}
			if(postb == POSTB_LOOP) // loop
			{
				while(myFrame >= duration)
					myFrame = myFrame - endFrame;
				
			}
		}
		
		lo = floor(myFrame);
		hi = hif = ceil(myFrame); // hif is for referencing data at hiframe, hi is for computing pct
		if(hif == duration)
			hif = 0;
		
		if(hi == lo) // if frame number is exactly on a frame e.g. not fractional
		{
			myPos = findDataAtFrame(lo,0);
			myRot = findDataAtFrame(lo,3);
			myScl = findDataAtFrame(lo,6);
		}
		
			
		if(lo != hi)
		{
			
			pct = (myFrame - lo) / (hi - lo);
			
			loPos = findDataAtFrame(lo,0);
			hiPos = findDataAtFrame(hif,0);
			loRot = findDataAtFrame(lo,3);
			hiRot = findDataAtFrame(hif,3);
	
			loScl = findDataAtFrame(lo,6);
			hiScl = findDataAtFrame(hif,6);
			myPos = ((hiPos - loPos) * pct) + loPos;
			myRot = ((hiRot - loRot) * pct) + loRot;
			myScl = ((hiScl - loScl) * pct) + loScl;
			
		}
		if(posx != false || posy != false || posz != false)
			ma.set(POSITION,filterMotion(myPos,1,time));
		if(roth != false || rotp != false || rotb != false)
			ma.set(ROTATION,filterMotion(myRot,2,time));
		if(sclx != false || scly != false || sclz != false)
			ma.set(SCALING,filterMotion(myScl,3,time));
	}
	else
	{
		if(fileexists(defFile))
		{
			f = File(defFile,"rb");
			if(!f)
			{
				return;
				//error(string("Error reading ",defFile));
			}
			//findFirstFrames();
			readM2L();
		}
	}
	
}

findDataAtFrame: myFrame, typeoffset
{
	if(f.offset() == -1)
		f.reopen("rb");
	finalOffset = INTEGER * 2; // objType and duration
	if(objType == 1) // camera type
		finalOffset = finalOffset + (DOUBLE * 5) + (INTEGER * 4); // camera related stuff

	finalOffset = finalOffset + (DOUBLE * duration); // time durations
	frameSize = 0;
	cameraTypeOffset = 0;
	if(objType == 1)
	{
		frameSize = frameSize + DOUBLE;
		cameraTypeOffset++;
		frameSize = frameSize + DOUBLE;
		cameraTypeOffset++;
		frameSize = frameSize + DOUBLE;
		cameraTypeOffset++;
		frameSize = frameSize + DOUBLE;
		cameraTypeOffset++;
	}	
	frameSize = frameSize + (DOUBLE * 9);
	
	finalOffset = finalOffset + (myFrame * frameSize); // find frame
	
	finalOffset = finalOffset + (DOUBLE * (typeoffset + cameraTypeOffset));
	f.offset(finalOffset,FROMSTART);
	
	if(typeoffset == 0) //position
	{
		a = f.readDouble();
		b = f.readDouble();
		c = -f.readDouble();
	}
	if(typeoffset == 3) //rotation
	{
		b = -f.readDouble();
		a = -f.readDouble();
		c = f.readDouble();
	}
	if(typeoffset == 6) // scale
	{
		a = f.readDouble();
		b = f.readDouble();
		c = f.readDouble();
	}
		
	//diag.post(myFrame,"  ",type,<a,b,c>," ",duration," ",f.name());

	return(<a,b,c>);

}
translateObjType: type
{
	if(type == 0)
		return("M2L Transform Only");
	if(type == 1)
		return("M2L Camera-Transform");
	if(type == nil)
		return("NONE");
}
destroy
{
	if(f)
		f.close();
}
refreshUI: val
{
	fn = val;
	openM2LFile(fn);
	setvalue(cstr_type,translateObjType(objType));
}
options
{
	
	reqbegin("Maya2LW2 Motion Options");
	reqsize(650,180);
	cfn_fn = ctlfilename("M2L File",defFile,100,1);
	cint_frameOffset = ctlinteger("Frame Offset",frameOffset);
	cpct_speed = ctlpercent("Speed",speed);
	cpu_preb = ctlpopup("Pre-behavior",preb,@"Dissolve Out","First Frame","Last Frame","Loop","Linear"@);
	cpu_postb = ctlpopup("Post-behavior",postb,@"Dissolve Out","First Frame","Last Frame","Loop","Linear"@);
	cstr_type = ctlstring("Type",translateObjType(objType),150);
	ccbx_px = ctlcheckbox("Position.X",posx);
	ccbx_py = ctlcheckbox("Position.Y",posy);
	ccbx_pz = ctlcheckbox("Position.Z",posz);
	
	ccbx_rx = ctlcheckbox("Rotation.H",roth);
	ccbx_ry = ctlcheckbox("Rotation.P",rotp);
	ccbx_rz = ctlcheckbox("Rotation.B",rotb);
	
	ccbx_sx = ctlcheckbox("Scale.X",sclx);
	ccbx_sy = ctlcheckbox("Scale.Y",scly);
	ccbx_sz = ctlcheckbox("Scale.Z",sclz);
	
	ctlposition(ccbx_px,250,24*1+5);
	ctlposition(ccbx_py,250,24*2+5);
	ctlposition(ccbx_pz,250,24*3+5);
	ctlposition(ccbx_rx,350,24*1+5);
	ctlposition(ccbx_ry,350,24*2+5);
	ctlposition(ccbx_rz,350,24*3+5);
	ctlposition(ccbx_sx,450,24*1+5);
	ctlposition(ccbx_sy,450,24*2+5);
	ctlposition(ccbx_sz,450,24*3+5);
                            
	
	ctlrefresh(cfn_fn,"refreshUI");
	return if(!reqpost());

	fn = getvalue(cfn_fn);
		
	frameOffset = getvalue(cint_frameOffset);
	speed = getvalue(cpct_speed);
	preb = getvalue(cpu_preb);
	postb = getvalue(cpu_postb);
	
	posx = getvalue(ccbx_px);
	posy = getvalue(ccbx_py);
	posz = getvalue(ccbx_pz);
	roth = getvalue(ccbx_rx);
	rotp = getvalue(ccbx_ry);
	rotb = getvalue(ccbx_rz);
	sclx = getvalue(ccbx_sx);
	scly = getvalue(ccbx_sy);
	sclz = getvalue(ccbx_sz);
	               
	openM2LFile(fn);
	
}
openM2LFile: fn
{
	if(!fileexists(fn))
		warn("File does not exist.");
	else
	{
		defFile = fn;
		(d1,d2) = split(fn);
		store("defFile",string(d1,d2));
		if(f)
			f.close();
		f = File(fn,"rb");
		if(!f)
			error("Error opening M2L file for reading.");
		else
			readM2L();
		
		// store first frame values

		
	}
	reqend();
}

findFirstFrames
{
	firstFramePos = findDataAtFrame(0,0);
	firstFrameRot = findDataAtFrame(0,3);
	firstFrameScl = findDataAtFrame(0,6);
}
readM2L
{
	
	sp = split(defFile);
	str = string("M2ML2MOT ",sp[3],sp[4]);
	setdesc(str);
	f.offset(0);
	objType = f.readInt();
	duration = f.readInt();
	if(objType == 1) // if camera m2l
	{
		apertureHeight = f.readDouble();
		isAnim_focalLength = f.readInt();
		focalLength = f.readDouble();
		isAnim_shutterAngle = f.readInt();
		shutterAngle = f.readDouble();
		isAnim_focalDistance = f.readInt();
		focalDistance = f.readDouble();
		isAnim_fStop = f.readInt();
		fStop = f.readDouble();
	}
	findFirstFrames();
	(t,d,startFrame,endFrame) = findM2LTopHeader(f);
	
}

token: myLine, myToken, mySlot, mode
{
	// mode 1: get mySlot only
	// mode 2: get mySlot and all to the right of it
	
	a = parse(myToken,myLine);
	
	if(mode == 1)
		return(a[mySlot]);
	if(mode == 2)
	{
		b = a[mySlot];
		for(i=mySlot+1;i<=a.size();i++)
		{
			b = myToken + a[i];
		}
		return(b);
	}
}
startsWith: myLine, myWord
{
	if(myWord == nil)
		return(nil);
	
	wordSize = myWord.size();
	a = strleft(myLine,wordSize);
	if(a == myWord)
		return(true);
	else
		return(false);
}
endsWith: myLine, myWord
{
	if(myWord == nil)
		return(nil);
	
	wordSize = myWord.size();
	a = strright(myLine,wordSize);
	if(a == myWord)
		return(true);
	else
		return(false);
}
load: what, io
{
	if(what == SCENEMODE)
	{
		line = io.read();
		
		if(line.contains("DEFFILE ") == false) // old version
		{
			defFile = line;
			line = io.read();
			frameOffset = integer(line);
			line = io.read();
			speed = number(line);
	//		preb = integer(io.read());
			line = io.read();
			postb = integer(line);
			str = io.read();
			posx = integer(str[1]);
			posy = integer(str[2]);
			posz = integer(str[3]);
			roth = integer(str[4]);
			rotp = integer(str[5]);
			rotb = integer(str[6]);
			sclx = integer(str[7]);
			scly = integer(str[8]);
			sclz = integer(str[9]);

		}
		else // new version
		{
			defFile = token(line," ",2,2);
			while(io.eof() == false && line != nil)
			{
				line = io.read();
				if(line == nil)
					continue;
				if(startsWith(line,"FRAMEOFFSET ") == true)
					frameOffset = integer(token(line," ",2,1));
				if(startsWith(line,"SPEED ") == true)
					speed = number(token(line," ",2,1));
				if(startsWith(line,"PREB ") == true)
					preb = integer(token(line," ",2,1));
				if(startsWith(line,"POSTB ") == true)
					postb = integer(token(line," ",2,1));
				if(startsWith(line,"POSTB ") == true)
					postb = integer(token(line," ",2,1));
				if(startsWith(line,"INCLUDECHANNELS ") == true)
				{
					str = token(line," ",2,2);
					posx = integer(str[1]);
					posy = integer(str[2]);
					posz = integer(str[3]);
					roth = integer(str[4]);
					rotp = integer(str[5]);
					rotb = integer(str[6]);
					sclx = integer(str[7]);
					scly = integer(str[8]);
					sclz = integer(str[9]);
				}
			}

		}
	}
}

save: what, io
{
	if(what == SCENEMODE)
	{
		io.writeln(string("DEFFILE ",defFile));
		io.writeln(string("FRAMEOFFSET ",frameOffset));
		io.writeln(string("SPEED ",speed));
		io.writeln(string("PREB ",preb));
		io.writeln(string("POSTB ",postb));
		str = string("INCLUDECHANNELS ",posx,posy,posz,roth,rotp,rotb,sclx,scly,sclz);
		io.writeln(str);

	}
}

findM2LTopHeader: fnf // start and end frames and finds duration
{
	fnf.rewind();
	t = fnf.readInt();
	d = fnf.readInt();
	if(t == 1)
		fnf.offset((DOUBLE * 5) + (INTEGER * 6));
	
	srcStart = fnf.readDouble() * Scene().fps;
	if(t == 1)
		fnf.offset((DOUBLE * 5) + (INTEGER * 6) + (DOUBLE * (d - 1)));
	else if(t == 0)
		fnf.offset(DOUBLE * d);
	srcEnd = fnf.readDouble() * Scene().fps;
	return(t,d,srcStart,srcEnd);
}