/*==================================================
Maya2LW[2]
Copyright (C) 2005  Lernie D. Ang

This file is part of Maya2LW[2]

Maya2LW is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
==================================================*/
@warnings 0

@define OA 1
@define POBJ 2
@define MOTOROLA 0
@define INTEL 1
particlesDir;
contentDir;
c1;

emitters;
lastVelocity;
lastPosition;

writeHeader: f, pc, numAttr
{
	f.write("PDC ");
	writeInt(f,1,MOTOROLA); // file format number, usually 1
	writeInt(f,1,MOTOROLA); // byte orer
	f.writeInt(0); // extra 1
	f.writeInt(0); // extra 2
	writeInt(f,pc,MOTOROLA); // particle count
	writeInt(f,numAttr,MOTOROLA); // number of attributes
	
	
}
writeDouble : outFile, x, pcOrder
{
    var byte[ 8 ];

    if ( x == 0 ) {
       byte[ 8 ] = 0;
       byte[ 7 ] = 0;
       byte[ 6 ] = 0;
       byte[ 5 ] = 0;
       byte[ 4 ] = 0;
       byte[ 3 ] = 0;
       byte[ 2 ] = 0;
       byte[ 1 ] = 0;
    }
    else
    {
       lgx = log( abs( x )) / log( 2.0 );
       e = integer( floor( lgx ));

       m = ( pow( 2.0, lgx - e ) - 1.0 ) * 1048576.0;
       m1 = integer( floor( m ));
       m = ( m - m1 ) * 65536.0;
       m2 = integer( floor( m ));
       m = ( m - m2 ) * 65536.0;
       m3 = integer( floor( m ));

       e += 1023;

       byte[ 1 ] = integer( e / 16 ) + ( x < 0 ? 128 : 0 );
       byte[ 2 ] = integer( m1 / 65536 ) + 16 * ( e % 16 );
       byte[ 3 ] = integer( m1 / 256 ) % 256;
       byte[ 4 ] = m1 % 256;
       byte[ 5 ] = m2 / 256;
       byte[ 6 ] = m2 % 256;
       byte[ 7 ] = m3 / 256;
       byte[ 8 ] = m3 % 256;
    }

    writeBytes( outFile, byte, 8, pcOrder );
}

writeInt : outFile, i, pcOrder
{
   var byte[ 4 ];

   if ( i >= 0 ) {
      byte[ 1 ] = integer( i / 16777216 );
      byte[ 2 ] = integer( i / 65536 ) % 256;
      byte[ 3 ] = integer( i / 256 ) % 256;
      byte[ 4 ] = i % 256;
   }
   else {
      j = -i - 1;
      byte[ 1 ] = 255 - integer( j / 16777216 );
      byte[ 2 ] = 255 - ( integer( j / 65536 ) % 256 );
      byte[ 3 ] = 255 - ( integer( j / 256 ) % 256 );
      byte[ 4 ] = 255 - ( j % 256 );
   }

   writeBytes( outFile, byte, 4, pcOrder );
}
writeBytes : outFile, byte, n, pcOrder
{
   if ( pcOrder ) {
      first = n;
      stop  = 0;
      step  = -1;
   }
   else {
      first = 1;
      stop  = n + 1;
      step  = 1;
   }

   for ( i = first; i != stop; i += step )
      outFile.writeByte( byte[ i ] );
}
list_count
{
	if(emitters == nil)
		return(nil);
	else
		return(emitters.size());
}

list_names: ndx
{
	return(emitters[ndx,OA].name);
}


acquireParticleObject: pemit
{
	if(pemit != nil)
	{
		Particle(pemit);
		if(sizeof(this) == 0)
			return(nil);
		else
		{
			(partObj) = Particle(pemit);
			return(partObj);
		}
	}
}

findEmitter
{
	emitters = nil;
	i = 0;
	m = Mesh();
	while(m)
	{
		if(p = acquireParticleObject(m))
		{
			emitters[++i,OA] = m;
			emitters[i,POBJ] = p; // particleobject
		}
		
		m = m.next();
	}
	
	
}

c_active: val
{
	return(false);
}

c_refresh: val
{
	if(strleft(val,1) != "\\")
		val = "\\" + val;

	setvalue(c1,val);
}

createDirs: val
{
	myDir = contentDir;
	dirsToCreate = parse("\\",val);
	for(i=1;i<=dirsToCreate.size();i++)
	{
		myDir = myDir + "\\" + dirsToCreate[i];
		if(!matchdirs(contentDir,myDir))
		{
			if(mkdir(myDir) == 1)
			{
				warn("Error creating directories.");
			}
		}
	}
}

generic // PFX to PDC
{
	

	findEmitter();

	if(emitters == nil)
		error("No emitters found in scene.");

	contentDir = getdir("Content");
	particlesDir = recall("particlesDir","\\particles");
	addSuffix = recall("addSuffix",1);
	
	temp[1] = 1;
	
	reqbegin("Maya2LW[2]: ParticleFX to Maya PDC");
	
	c0 = ctlstring("Content Directory",contentDir,400);
	c1 = ctlstring("Particle Sub-Dir",particlesDir,400);
	c2 = ctlcheckbox("Add 'Shape' Suffix",addSuffix);
	d1 = ctllistbox("Emitters",400,200,"list_count","list_names");
	
	ctlactive(c1,"c_active",c0);
	ctlrefresh(c1,"c_refresh");

	return if(!reqpost());

	particlesDir = getvalue(c1);
	addSuffix = getvalue(c2);
	se = getvalue(d1);
	
	if(se == nil)
		error("Nothing selected.");

	createDirs(particlesDir);

	reqend();
	
	startFrame = Scene().previewstart;
	endFrame = Scene().previewend;
	
	for(h=startFrame;h<=endFrame;h++)
	{
		GoToFrame(h);
		filenum = string(h * 200); // for maya file numbering of PDC format
		for(i=1;i<=se.size();i++)
		{
			ndx = se[i];
			po = emitters[ndx,POBJ]; // po is particle object
			particleCount = po.count;
			if(addSuffix == true)
				fname = contentDir + particlesDir + "\\" + emitters[ndx,OA].name + "Shape." + filenum + ".pdc";
			else
				fname = contentDir + particlesDir + "\\" + emitters[ndx,OA].name + "." + filenum + ".pdc";

			fn = File(fname,"wb");
			if(!fn)
				error("Error opening file for writing.");

			// write header
			
			writeHeader(fn,particleCount,9);

			writeInt(fn,8,MOTOROLA);
			fn.write("position");
			writeInt(fn,5,MOTOROLA); // vector array	
			for(j=particleCount;j>=1;j--)
			{
				pos = po.getParticle(j,PART_POSITION);						
				writeDouble(fn,-pos.x,MOTOROLA);
				writeDouble(fn,pos.y,MOTOROLA);
				writeDouble(fn,pos.z,MOTOROLA);

				id = po.getParticle(j,PART_ID);
				str = string(id,"--",h); 

				lastPosition[str] = vel; // assoc arrays

			}

			writeInt(fn,12,MOTOROLA);
			fn.write("lastPosition");
			writeInt(fn,5,MOTOROLA); // vector array
			for(j=particleCount;j>=1;j--)
			{
				id = po.getParticle(j,PART_ID);
				lastPos = <0,0,0>;
				str = string(id,"--",h - 1);
				
				if(lastPosition[str] != nil)
				{
					lastPos = lastPosition[str];
				}

				writeDouble(fn,-lastPos.x,MOTOROLA);
				writeDouble(fn,lastPos.y,MOTOROLA);
				writeDouble(fn,lastPos.z,MOTOROLA);
			}

			writeInt(fn,10,MOTOROLA);
			fn.write("particleId");
			writeInt(fn,3,MOTOROLA); // doubles array
			for(j=particleCount;j>=1;j--)
			{
				id = po.getParticle(j,PART_ID);
				writeDouble(fn,id,MOTOROLA);
			}

			writeInt(fn,4,MOTOROLA);
			fn.write("mass");
			writeInt(fn,3,MOTOROLA); // doubles array
			for(j=particleCount;j>=1;j--)
			{
				writeDouble(fn,1,MOTOROLA);
			}

			writeInt(fn,8,MOTOROLA);
			fn.write("velocity");
			writeInt(fn,5,MOTOROLA); // vector array				
			for(j=particleCount;j>=1;j--)
			{
				vel = po.getParticle(j,PART_VELOCITY) * 1000;
				writeDouble(fn,-vel.x,MOTOROLA);
				writeDouble(fn,vel.y,MOTOROLA);
				writeDouble(fn,vel.z,MOTOROLA);

				id = po.getParticle(j,PART_ID);
				str = string(id,"--",h); 

				lastVelocity[str] = vel; // assoc arrays
			}

			writeInt(fn,12,MOTOROLA);
			fn.write("lastVelocity");
			writeInt(fn,5,MOTOROLA); // vector array
			for(j=particleCount;j>=1;j--)
			{
				id = po.getParticle(j,PART_ID);
				lastVel = <0,0,0>;
				str = string(id,"--",h - 1);
				
				if(lastVelocity[str] != nil)
				{
					lastVel = lastVelocity[str];
				}

				writeDouble(fn,-lastVel.x,MOTOROLA);
				writeDouble(fn,lastVel.y,MOTOROLA);
				writeDouble(fn,lastVel.z,MOTOROLA);
			}

			writeInt(fn,12,MOTOROLA);
			fn.write("acceleration");
			writeInt(fn,5,MOTOROLA); // vector array
			for(j=particleCount;j>=1;j--)
			{
				id = po.getParticle(j,PART_ID);

				acc = <0,0,0>;
				str = string(id,"--",h - 1);
				if(lastVelocity[str] != nil)
				{
					lastVel = lastVelocity[str];
					vel = po.getParticle(j,PART_VELOCITY) * 1000;
					acc = (vel - lastVel) / (1 / Scene().fps);
				}

				writeDouble(fn,-acc.x,MOTOROLA);
				writeDouble(fn,acc.y,MOTOROLA);
				writeDouble(fn,acc.z,MOTOROLA);
			}



			writeInt(fn,3,MOTOROLA);
			fn.write("age");
			writeInt(fn,3,MOTOROLA); // doubles array
			for(j=particleCount;j>=1;j--)
			{
				age = po.getParticle(j,PART_AGE);
				writeDouble(fn,age,MOTOROLA);
			}

			writeInt(fn,9,MOTOROLA);
			fn.write("birthTime");
			writeInt(fn,3,MOTOROLA); // doubles array
			curtime = Scene().currenttime;
			for(j=particleCount;j>=1;j--)
			{
				age = po.getParticle(j,PART_AGE);
				birthTime = curtime - age;
				writeDouble(fn,birthTime,MOTOROLA);
			}
			
		}
	
	}
	store("particlesDir",particlesDir);
	store("addSuffix",addSuffix);
}

