/*==================================================
Maya2LW[2]
Copyright (C) 2005  Lernie D. Ang

This file is part of Maya2LW[2]

Maya2LW is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
==================================================*/

@version 2.6
@warnings 0
@script generic
@define VERSIONNAME "Maya2LW[2] Import M2L"


generic
{
	sel = Scene().getSelect();
	if(!sel)
		error("No selection to process.");
	contentDir = getdir("Content");

	workDir = recall("workDir",contentDir);


	file = getfile("Load M2L File","*.m2l",contentDir,true);
	
	f = File(file,"rb");
	
	if(!f)
		error("Error opening file for reading.");

	objType = f.readInt();
	duration = f.readInt();
	for(i=1;i<=sel.size();i++)
		applyM2L(sel[i],f,duration,objType);

	f.close();
	
	(d,p,f,e) = split(file);
	
	workDir = string(d,p);
	
	store("workDir",workDir);
	
}
applyM2L: thing, f, dur, type
{
	genus = thing.genus;
	SelectItem(thing.id);

	if(type == 1 && genus == CAMERA) // if file type camera
	{
		apertureHeight = f.readDouble();
		apertureHeight = apertureHeight * .0254;
		str = string("ApertureHeight ",apertureHeight);
		CommandInput(str);
	
		isAnim_focalLength = f.readInt();
		focalLength = f.readDouble();
		fl2 = (focalLength * 2) / apertureHeight;
		str = string("CommandInput ZoomFactor ",fl2 / 1000);
		CommandInput("RemoveEnvelope ZoomFactor");
		CommandInput(str); 
		if(isAnim_focalLength == 1)
			CommandInput("AddEnvelope ZoomFactor");

		
		isAnim_shutterAngle = f.readInt();
		shutterAngle = f.readDouble();
		sa2 = shutterAngle / 360;
		str = string("CommandInput BlurLength ",sa2);
		CommandInput("RemoveEnvelope BlurLength");
		CommandInput(str);
		if(isAnim_shutterAngle == 1)
			CommandInput("AddEnvelope BlurLength");
		
		isAnim_focalDistance = f.readInt();
		focalDistance = f.readDouble();
		str = string("CommandInput FocalDistance ",focalDistance);
		CommandInput("RemoveEnvelope FocalDistance");
		CommandInput(str);
		if(isAnim_focalDistance == 1)
			CommandInput("AddEnvelope FocalDistance");
		
		isAnim_fStop = f.readInt();
		fStop = f.readDouble();
		str = string("CommandInput LensFStop ",fStop);
		CommandInput("RemoveEnvelope FStop");
		CommandInput(str);
		if(isAnim_fStop == 1)
			CommandInput("AddEnvelope FStop");	
	}

	//	get oa's chane
	c = thing.firstChannel();
	while(c)
	{
		kc = c.keyCount;
		for(ii=1;ii<=kc;ii++)
			c.deleteKey(c.keys[1]);

		c.preBehavior = CHAN_CONSTANT;

		c.postBehavior = CHAN_CONSTANT;

		if(c.name == "Position.X")
			chan_posx = c;
		if(c.name == "Position.Y")
			chan_posy = c;
		if(c.name == "Position.Z")
			chan_posz = c;
		if(c.name == "Rotation.H")
			chan_roth = c;
		if(c.name == "Rotation.P")
			chan_rotp = c;
		if(c.name == "Rotation.B")
			chan_rotb = c;
		if(c.name == "Scale.X")
			chan_sclx = c;
		if(c.name == "Scale.Y")
			chan_scly = c;
		if(c.name == "Scale.Z")
			chan_sclz = c;
		if(c.name == "ZoomFactor")
			chan_focallength = c;
		if(c.name == "BlurLength")
			chan_shutterangle = c;
		if(c.name == "FocalDistance")
			chan_focaldistance = c;
		if(c.name == "FStop")
			chan_fstop = c;
			
		c = thing.nextChannel();
	}

	
	for(i=1;i<=dur;i++)
		timeindices[i] = f.readDouble(); // get time indices

	
	firstKey = true;


	for(i=1;i<=timeindices.size();i++)
	{
		ti = timeindices[i];
		if(f.eof() == true)
			return;
		if(type == 1) // if file type camera
		{
			//if(isAnim_focalLength == 1)
				fl = f.readDouble();	
			//if(isAnim_shutterAngle == 1)
				sa = f.readDouble();
			//if(isAnim_focalDistance == 1)
				fd = f.readDouble();
			//if(isAnim_fStop == 1)
				fs = f.readDouble();
		}

		px = f.readDouble();
		py = f.readDouble();
		pz = f.readDouble();


		valmatrixUp = <0,0,0>;
		valmatrixRight = <0,0,0>;
		valmatrixForward = <0,0,0>;

		rx = f.readDouble();
		ry = f.readDouble();
		rz = f.readDouble();
		
		sx = f.readDouble();
		sy = f.readDouble();
		sz = f.readDouble();


		if(type == 1) // if file type camera
		{
			if(isAnim_focalLength == 1)
			{
				if(chan_focallength)
				{
					fl2 = (fl * 2) / apertureHeight;
					a = chan_focallength.createKey(ti,fl2 / 1000);
					if(firstKey)
						chan_focallength.setKeyCurve(a,CHAN_STEPPED);
					
				}
			}
			if(isAnim_shutterAngle == 1)
			{
				sa2 = sa / 360;
				if(chan_shutterangle)
					a = chan_shutterangle.createKey(ti,sa2);
				if(firstKey)
					chan_shutterangle.setKeyCurve(a,CHAN_STEPPED);

			}
			if(isAnim_focalDistance == 1)
			{
				if(chan_focaldistance)
					a = chan_focaldistance.createKey(ti,fd);
				if(firstKey)
					chan_focaldistance.setKeyCurve(a,CHAN_STEPPED);

			}	
			if(isAnim_fStop == 1)
			{
				if(chan_fstop)
					a = chan_fstop.createKey(ti,fs);
				if(firstKey)
					chan_fstop.setKeyCurve(a,CHAN_STEPPED);

			}
		}
				
		if(chan_posx)
		{
			a = chan_posx.createKey(ti,px);
			if(firstKey)
				chan_posx.setKeyCurve(a,CHAN_STEPPED);
		}

		if(chan_posy)
		{
			a = chan_posy.createKey(ti,py);
			if(firstKey)
				chan_posy.setKeyCurve(a,CHAN_STEPPED);
		}


		if(chan_posz)
		{
			a = chan_posz.createKey(ti,-pz);
			if(firstKey)
				chan_posz.setKeyCurve(a,CHAN_STEPPED);
		}

		valvec = <0,0,0>;
		valvec.x = rx;
		valvec.y = ry;
		valvec.z = rz;
		

		if(chan_roth)
		{
			a = chan_roth.createKey(ti,-valvec.y);
			if(firstKey)
				chan_roth.setKeyCurve(a,CHAN_STEPPED);
		}
		if(chan_rotp)
		{
			a = chan_rotp.createKey(ti,-valvec.x);
			if(firstKey)
				chan_rotp.setKeyCurve(a,CHAN_STEPPED);
		}
		if(chan_rotb)
		{
			a = chan_rotb.createKey(ti,valvec.z);
			if(firstKey)
				chan_rotb.setKeyCurve(a,CHAN_STEPPED);
		}

		if(chan_sclx)
		{
			a = chan_sclx.createKey(ti,sx);
			if(firstKey)
				chan_sclx.setKeyCurve(a,CHAN_STEPPED);
		}
		
		if(chan_scly)
		{
			a = chan_scly.createKey(ti,sy);
			if(firstKey)
				chan_scly.setKeyCurve(a,CHAN_STEPPED);
		}
		
		if(chan_sclz)
		{
			a = chan_sclz.createKey(ti,sz);
			if(firstKey)
				chan_sclz.setKeyCurve(a,CHAN_STEPPED);
		}
		firstKey = false;
	}	
}


/*======================================================================================================
the code below by Stuart Aitken, Axis Animation
======================================================================================================*/
matrix_mult: A, B
{
    C[1] = <(A[1].x*B[1].x)+(A[1].y*B[2].x)+(A[1].z*B[3].x),(A[1].x*B[1].y)+(A[1].y*B[2].y)+(A[1].z*B[3].y),(A[1].x*B[1].z)+(A[1].y*B[2].z)+(A[1].z*B[3].z)>;

    C[2] = <(A[2].x*B[1].x)+(A[2].y*B[2].x)+(A[2].z*B[3].x),(A[2].x*B[1].y)+(A[2].y*B[2].y)+(A[2].z*B[3].y),(A[2].x*B[1].z)+(A[2].y*B[2].z)+(A[2].z*B[3].z)>;

    C[3] = <(A[3].x*B[1].x)+(A[3].y*B[2].x)+(A[3].z*B[3].x),(A[3].x*B[1].y)+(A[3].y*B[2].y)+(A[3].z*B[3].y),(A[3].x*B[1].z)+(A[3].y*B[2].z)+(A[3].z*B[3].z)>;

    return C;
}
vec_HPB: mat
{
	//normalises vector then uses trig to solve H,P,B
	l = normalize(mat[1]);
	m = normalize(mat[2]);
	n = normalize(mat[3]);
	rotP = asin(-n.y);

	x = n.z/cos(rotP);
	x = max(-1.0,x);
	x = min(1.0,x);
	rotH = acos(x);
	rotH = (2*PI - rotH) if (n.x<0.0);

	x = m.y/cos(rotP);
	x = max(-1.0,x);
	x = min(1.0,x);
	rotB = acos(x);
	rotB = (2*PI - rotB) if (l.y<0.0);
	rot = <deg(rotH),deg(rotP),deg(rotB)>;
	rot.x = rot.x - 360 if rot.x > 180;
	return(rot);
}
