@warnings 0
myNameList;
objDir; c2;
vmapOA;
choiceEdgeDataFile;
edgeDataFile;
exportUV;
defUnits;
metricUnits = @"millimeter","centimeter","meter","kilometer","inch","foot","yard","mile"@;
mesh;
// version 1.1: added edge data mode and edge data file to speed up exporting on meshes whose topology does not change
//					 instructions: user uses the "Save" function to export an edge data file that is created along with the usual export. script stores calculated edges in a file.
//					 later, the user selects "Load" and specifies the edge data file and exports as normal. script will be faster when referencing the edge data file, because no extra calculation will be performed.
// version 1.2: added morph export convenience feature

list_count
{
	return(vmapOA.size());
}
list_names: ndx
{
	return(vmapOA[ndx].name);
}

chooseOutFilename
{
	fn = getfile("Maya2LW[2] Export Maya ASCII","*.ma",objDir);
	
	
	
	if(fn)
	{
		setvalue(c2,fn);
		fns = split(fn);
		store("objDir",fns[1] + fns[2]);
	}
}
main
{
	//pc = editbegin() || error("No data.");
	exportUV = integer(recall("exportUV",0));
	defUnits = integer(recall("defUnits",1));
	choiceEdgeDataFile = integer(recall("choiceEdgeDataFile",3));
	
	myNameList[1] = nil;
	if(pointcount() == 0)
		error("No data.");
	mesh = Mesh(0);
	if(!mesh)
		error("No mesh.");
	objDir = recall("objDir",getdir("Objects"));
	if(objDir.contains(getdir("Content")) == false)
		objDir = getdir("Objects");
	edgeDataFile = recall("edgeDataFile",string(objDir,"\\*.txt"));
	exportMorphs = integer(recall("exportMorphs",1));
	
	// get point list
	// get poly list
	
	type = VMMORPH;
	vmap = VMap(type);
	while(vmap && vmap.type == type)
	{
		vmapnames += vmap.name;
		vmapOA += vmap;
		vmap = vmap.next();
	}
	
	
	reqbegin("Maya2LW[2] Maya ASCII Exporter 1.2");
	c1 = ctlbutton("Choose output filename",150,"chooseOutFilename");
	c2 = ctlstring("Output Filename",string(objDir,"\\*.ma"),500);
	c2a = ctlchoice("Edge Data Mode",choiceEdgeDataFile,@"Save","Load","Do not use"@);
	c2b = ctlfilename("Edge Data File",edgeDataFile,100);
	c3 = ctlcheckbox("Export UVs",exportUV);
	c4 = ctlpopup("Default Units",defUnits,metricUnits);
	d1 = ctlcheckbox("Export Morphs",exportMorphs);
	d2 = ctllistbox("Morph Maps",500,200,"list_count","list_names");
	chooseOutFilename();
	if(!reqpost())
	{
		warn("User aborted operation.");
		return;
	}
	fn = getvalue(c2);
	choiceEdgeDataFile = getvalue(c2a);
	edgeDataFile = getvalue(c2b);
	exportUV = getvalue(c3);
	defUnits = getvalue(c4);
	exportMorphs = getvalue(d1);
	selectedMorphs = getvalue(d2);
	store("exportUV",exportUV);
	store("defUnits",defUnits);
	store("edgeDataFile",edgeDataFile);
	store("choiceEdgeDataFile",choiceEdgeDataFile);
	store("exportMorphs",exportMorphs);

	myTime1 = time();
	if(exportMorphs == true)
	{
		// fn is full path to the file
		
		for(i=1;i<=selectedMorphs.size();i++)
		{
			// go to the morph now
			ndx = selectedMorphs[i];
			goToMorphMap(vmapOA[ndx]);
			myName = vmapOA[ndx].name;
			if(strright(fn,3) == ".ma") // strip the .ma first
				fn = strleft(fn,fn.size() - 3);

			fnNew = fn + "." + myName + ".ma";
			mainExportProcess(fnNew);
			fnNew = nil;
		}
	}
	else
	{
		mainExportProcess(fn);
	}

	myTime2 = time();
	eTicks = myTime2[4] - myTime1[4];
	eMins = eTicks / 60;
	info(string("DONE. ",eTicks," seconds elapsed \(",eMins,"minutes)."));
}

mainExportProcess: fn
{
	if(strright(fn,3) != ".ma")
		fn = fn + ".ma";

	(d,p,f) = split(fn);

	f = File(fn,"w");
	
		
	
	if(!f)
		error("Unable to open file for writing.");
	
	selmode(GLOBAL);	
	fgl = lyrfg();
	writeHeader(f,metricUnits[defUnits]);
	i = 0;

	if(exportUV == 1)
	{
		vmoa = VMap(VMTEXTURE);
		while(vmoa)
		{
			if(vmoa.type == VMTEXTURE)
			{
				if(vmoa != nil)
				{
					vmap += vmoa;
				}		
				vmoa = vmoa.next();
			}
			else
				vmoa = vmoa.next();
		}
	}
	else
		vmap = nil;
	
	sni = 0; // shape nodes iterator
	for(h=1;h<=fgl.size();h++) // PER LAYER
	{
		lyrsetfg(fgl[h]);
		myName = string(mesh.layerName(fgl[h]));
		if(myName == "")
			myName = string("Layer",fgl[h]);
		checkDuplicateNames(myName);
		myName2 = this;
		if(choiceEdgeDataFile == 1)
		{
			edf = split(edgeDataFile);
			edfFin = string(edf[1],"\\",edf[2],"\\",edf[3],".",myName,edf[4]);
			ef = File(edfFin,"w");
			if(!ef)
				error("Unable to open Edge Data File for writing.");
		}
		if(choiceEdgeDataFile == 2)
		{
			edf = split(edgeDataFile);
			edfFin = string(edf[1],"\\",edf[2],"\\",edf[3],".",myName,edf[4]);
			ef = File(edgeDataFile,"r");
			if(!ef)
				error("Unable to open Edge Data File for reading.");
		}


		
		
		

		writeTransformNode(f,myName2);
		
		pc = editbegin();
		shapeNodes[++sni] = myName2 + "Shape";
		writeShapeNode(f,myName2,pc);
		
		
		i = 0;
		k = 0;
		abort = false;
		cols = 1;
		monStr = string("Processing layer ",h,": ",myName2);
		warn(monStr);
		moninit(pc,monStr);
		// POINTS
		while(i<=pc)
		{
			if(monstep())
				return;
			if((i + 1) > pc)
				break;
			firstPoint = k * 150;
			if(firstPoint > (pc - 1))
				firstPoint = (pc - 1);

			lastPoint = ((k + 1) * 150) - 1;
			if(lastPoint > (pc - 1))
				lastPoint = (pc - 1);
			
			if(firstPoint == lastPoint)
				string("\tsetAttr ","\".ed[",lastPoint,"]\"");
			string("\tsetAttr ","\".vt[",firstPoint,":",lastPoint,"]\"");
			f.writeln(this);
			f.write("\t\t");
			
			for(j=1;j<=150;j++)
			{
				i++;				
				if(i>pc)
				{
					abort = true;
					break;
				}
				// find UV
				
				for(m=1;m<=vmap.size();m++)
				{
					if(uvpntcnt`m == nil)
						uvpntcnt`m = 0;
					uvpc = 0; // point counter for each vmap
					if(vmap[m].isMapped(points[i])) // uvpnts`vmaps["point id"] = pointindex
					{
						myPointIDStr = string(integer(points[i]));
						uvpntcnt`m = uvpntcnt`m + 1;
						uvpntsID`m[uvpntcnt`m] = myPointIDStr;
						uvpnts`m[myPointIDStr] = uvpntcnt`m - 1;
						uvpntsVal`m[myPointIDStr] = <vmap[m].getValue(points[i],1),vmap[m].getValue(points[i],2),points[i]>;
						
						dPoly = points[i].polygon(); // check for dis
						
						for(s=1;s<=dPoly.size();s++)
						{
							if(dPoly[s] == nil)
								continue;
							if(vmap[m].getValue(points[i],dPoly[s],1) != nil)
							{
								myPointIDStr = string(integer(points[i]),"_D"); // signifies discon uvs
								uvpntcnt`m = uvpntcnt`m + 1;
								uvpntsID`m[uvpntcnt`m] = myPointIDStr;
								uvpnts`m[myPointIDStr] = uvpntcnt`m - 1; // uvpnts`m is the index used for face uv recovery
								uvpntsVal`m[myPointIDStr] = <vmap[m].getValue(points[i],dPoly[s],1),vmap[m].getValue(points[i],dPoly[s],2),points[i]>;
							}
						}

					}
				}				
				x = points[i].x;
				y = points[i].y;
				z = points[i].z;
				if(cols > 3)
				{
					string(x," ",y," ",-z,"\n\t\t");
					f.write(this);
					cols = 1;
				}
				else
				{
					string(x," ",y," ",-z," ");
					f.write(this);
					cols++;
				}
			}
			if(abort == true)
			{
				f.write("\;");
				break;
			}
			k++;
			f.writeln("\;");
		}

	// UVS
		monend();
		
		moninit(vmap.size(),"Processing VMaps");
		warn("Processing VMaps");
		for(i=1;i<=vmap.size();i++) // go through every vmap uvpnts[vmap,point,SLOT]
		{
			if(monstep())
				return;
			ui = 0;
			k = 0;
			abort = false;
			cols = 1;	
			uvSetName = vmap[i].name;
			uvPointCount = uvpntcnt`i;
			if(uvPointCount == 0)
				continue;
			mNdx = i - 1;
			
			f.nl();
			
			string("\tsetAttr \".uvst[",mNdx,"].uvsn\" -type \"string\" \"",uvSetName,"\"\;");
			f.writeln(this);
			//setAttr -s 969 ".uvst[2].uvsp";
			string("\tsetAttr -s ",uvPointCount," \".uvst[",mNdx,"].uvsp\"\;");
			f.writeln(this);
			f.write("\t\t");
			//setAttr -s 38 ".uvst[0].uvsp[0:37]" -type "float2" -7.2932471e-007
			while(ui<=uvPointCount)
			{
				if((ui + 1) > uvPointCount)
					break;
				firstUV = k * 150;
				if(firstUV > (uvPointCount - 1))
					firstUV = (uvPointCount - 1);
		
				lastUV = ((k + 1) * 150) - 1;
				if(lastUV > (uvPointCount - 1))
					lastUV = (uvPointCount - 1);
				
				if(firstUV == lastUV)
					string("\tsetAttr \".uvst[",mNdx,"].uvsp[",firstUV,"]\" -type \"float2\" ");
				else	
					string("\tsetAttr \".uvst[",mNdx,"].uvsp[",firstUV,":",lastUV,"]\" -type \"float2\" ");
				f.write(this);
				for(j=1;j<=150;j++)
				{
					ui++;
					if(ui > uvPointCount)
						break;
					
					uvpntsID`i[ui];
					myVec = uvpntsVal`i[this];
					uValue = myVec.x;
					vValue = myVec.y;
					if(cols > 3)
					{
						string(uValue," ",vValue,"\n\t\t");
						f.write(this);
						
						cols = 1;
					}
					else
					{
						string(uValue," ",vValue," ");
						f.write(this);
						cols++;
					}
				}
				k++;
				f.writeln("\;");
			}
		}				
	
		monend();
	// FACES / EDGES
		i = 0;
		polyEdge = nil;
		
		polys = nil;
		
		foreach(x,polygons)
			polys[++i] = x;
		edges = nil;
		edgeNames = nil;
		edges["nil"] = nil;
		ei = -1; // edge count index
		

		if(choiceEdgeDataFile == 2) // load
		{
			moninit(ef.linecount(),"Reading component data from file");
			warn("Reading component data from file");
			while(ef.eof() == false)
			{
				if(monstep())
					return;
				pl = nil;
				line = ef.read();
				if(line == nil)
					warn(ef.linecount());
				pl = parse("_",line);
				
				if(pl[1] == "EN")
				{
					eni = integer(pl[2]);
					enval = string(pl[3]);
					edgeNames[eni] = enval;
					
				}
				if(pl[1] == "PE")
				{
					peiA = integer(pl[2]);
					peiB = integer(pl[3]);
					peval = integer(pl[4]);
					polyEdge[peiA,peiB] = peval;
				}
				
			}
			polyEdge.pack();
			polyEdge.trunc();
		}
		else if(choiceEdgeDataFile == 1 || choiceEdgeDataFile == 3)
		{
			moninit(polys.size(),"Collecting component data");
			warn("Collecting component data...");
		
			for(i=1;i<=polys.size();i++)
			{
				if(monstep())
					return;
				pe = 0;
				// collect edges and UV
				
				for(j=1;j<=polys[i].pointCount;j++)
				{
					
					point1 = polys[i].points[j];
					
					if(j == polys[i].pointCount)
						point2 = polys[i].points[1];
					else
						point2 = polys[i].points[j+1];
					
					point1Str = string(integer(point1) - 1);
					point2Str = string(integer(point2) - 1);
					
					// define the edge both ways
					thisEdgeStr1 = string(point1Str,"-",point2Str);
					thisEdgeStr2 = string(point2Str,"-",point1Str);
					
					// check to see if the edge is the the edges array[]
					// edges[] contain pairs of points defining an edge expressed in zero-based pointindex
					// edgeNames[] contain pairs of points defining an edge expressed in STRINGS
					// polyEdge[] is multidim: each poly with corresponding edges which is expressed in edge index
					if(edges[thisEdgeStr1] == nil && edges[thisEdgeStr2] == nil) // if no such edge exists
					{
						
						
						// put newly found edge into arrays
						edges[thisEdgeStr1] = ++ei;
						edgeNames[ei+1] = thisEdgeStr1;
						polyEdge[i,++pe] = ei;
						// need to save out edgeNames polyFace, polyEdge
						if(choiceEdgeDataFile == 1) //save
						{
							ef.writeln("EN_",ei+1,"_",thisEdgeStr1);
							ef.writeln("PE_",i,"_",pe,"_",ei);
						}
						continue;
					}
					if(edges[thisEdgeStr1] != nil) // if edge exists in correct direction
					{
						polyEdge[i,++pe] = edges[thisEdgeStr1];
						if(choiceEdgeDataFile == 1) //save
						{
							ef.writeln("PE_",i,"_",pe,"_",edges[thisEdgeStr1]);
						}
						continue;
					}
					if(edges[thisEdgeStr2] != nil) // if edge exists in reverse direciton
					{
						polyEdge[i,++pe] = edges[thisEdgeStr2];
						if(choiceEdgeDataFile == 1) //save
							ef.writeln("PE_",i,"_",pe,"_",edges[thisEdgeStr2]);
						continue;
					}
				}
			}
			
			monend();
		}
		// process edges
		string("\t","setAttr -s ",edgeNames.size()," \".ed\"\;"); f.writeln(this);
	
		i = 0;
		k = 0;
		abort = false;
		cols = 1;
		
		moninit(polys.size(),"Processing Edges");
		warn("Processing Edges");
		while(i<=edgeNames.size())
		{
			if(monstep())
				return;
			if((i+1) > edgeNames.size())
				break;
			firstEdge = k * 150;
			if(firstEdge > (edgeNames.size() - 1))
				firstEdge = (edgeNames.size() - 1);
	
			lastEdge = ((k + 1) * 150) - 1;
			if(lastEdge > (edgeNames.size() - 1))
				lastEdge = (edgeNames.size() - 1);
			if(firstEdge == lastEdge)
				string("\tsetAttr ","\".ed[",lastEdge,"]\"");
			else
				string("\tsetAttr ","\".ed[",firstEdge,":",lastEdge,"]\"");
			f.writeln(this);
			f.write("\t\t");
			for(j=1;j<=150;j++)
			{
				i++;				
				if(i>edgeNames.size())
				{
					abort = true;
					break;
				}
				if(edgeNames[i] == nil)
					info(i,"  ",edgeNames[i]);
				(edge1,edge2) = parse("-",edgeNames[i]);
				
				if(cols > 3)
				{
					string(edge1," ",edge2," ",0,"\n\t\t");
					f.write(this);
					cols = 1;
				}
				else
				{
					string(edge1," ",edge2," ",0," ");
					f.write(this);
					cols++;
				}
			}
			if(abort == true)
			{
				f.write("\;");
				break;
			}
			k++;
			f.writeln("\;");
		}
		monend();
		// process polys

		string("\t"," setAttr -s ",polyEdge.size()," \".fc\"\;"); f.writeln(this);
	
		i = 0;
		k = 0;
		abort = false;
		cols = 1;
		moninit(polys.size(),"Processing PolyEdges");
		warn("Processing PolyEdges");	
		while(i<=polyEdge.size())
		{
			if(monstep())
				return;
			if((i+1) > polyEdge.size())
				break;
			firstPoly = k * 150;
			if(firstPoly > (polyEdge.size() - 1))
				firstPoly = (polyEdge.size() - 1);
	
			lastPoly = ((k + 1) * 150) - 1;
			if(lastPoly > (polyEdge.size() - 1))
				lastPoly = (polyEdge.size() - 1);
			if(firstPoly == lastPoly)
				string("\tsetAttr ","\".fc[",lastPoly,"]\""," -type \"polyFaces\"");
			else
				string("\tsetAttr ","\".fc[",firstPoly,":",lastPoly,"]\""," -type \"polyFaces\"");
			f.writeln(this);
			for(j=1;j<=150;j++)
			{
				i++;				
				if(i>polyEdge.size())
				{
					abort = true;
					break;
				}

				str = string("f ",polyEdge[i].size());
				str2 = "";
				
				for(l=1;l<=polyEdge[i].size();l++) // for every edge
				{
					str = str + " " + polyEdge[i,l];
				}
				f.writeln(str);	

				myPoly = polys[i]; // polyid
				for(a=1;a<=vmap.size();a++) // for every vmap
				{
					hasVMap = false;
					str2 = string(str2,"mu ",a - 1," ",myPoly.pointCount);
					for(b=1;b<=myPoly.pointCount;b++) // for every point
					{
						myPoint = myPoly.points[b];
						if(vmap[a].isMapped(myPoint))
						{
							hasVMap = true;
							myPointIDStr = string(integer(myPoint));
							ndx = uvpnts`a[string(myPointIDStr)];
						}
						if(vmap[a].isMapped(myPoint,myPoly)) //check for discont uvs
						{
							hasVMap = true;
							myPointIDStr = string(integer(myPoint),"_D");
							ndx = uvpnts`a[myPointIDStr];
						}
						str2 = str2 + " " + ndx;
					}
					
					if(hasVMap == true)
					{
						str2 = str2 + "\n ";
						f.writeln(str2);
					}
					if(hasVMap == false)
						str2 = "";
				}
				
			}
			if(abort == true)
			{
				f.write("\;");
				break;
			}
			k++;
			f.writeln("\;");
		}
		monend();
		str = string("setAttr -s 2 \".iog[0].og\"\;");
		f.writeln(str);
		editend();
	}	

	writeTail(f,shapeNodes);


	f.close();	
}
checkDuplicateNames: thisName
{
	if(myNameList.contains(thisName))
	{
		
		i = 1;
		while(myNameList.contains(string(thisName,i)))
			i++;
		thisName = string(thisName,i);
		return(thisName);
	}
	else
		return(thisName);
}
writeTail: f, snodes
{
	f.writeln(string("createNode lightLinker -n \"lightLinker1\"\;"));
	f.writeln(string("createNode materialInfo -n \"materialInfo1\"\;"));
	f.writeln(string("createNode shadingEngine -n \"Default1SG\"\;"));
	f.writeln(string("createNode lambert -n \"Default1\"\;"));
	f.writeln(string("setAttr \".c\" -type \"float3\" 0.75 0.75 0.75\;"));
	f.writeln(string("setAttr \".dc\" 1 \;"));
	//f.writeln(string("setAttr \".sc\" 0 0 0 \;"));
	//f.writeln(string("setAttr \".cp\" 32 \;"));
	f.writeln(string("connectAttr \"lightLinker1.msg\" \":lightList1.ln\" -na\;"));
	f.writeln(string("connectAttr \":defaultLightSet.msg\" \"lightLinker1.lnk[0].llnk\"\;"));
	f.writeln(string("connectAttr \"Default1SG.msg\" \"materialInfo1.sg\"\;"));
	f.writeln(string("connectAttr \"Default1.oc\" \"Default1SG.ss\"\;"));
	f.writeln(string("connectAttr \"Default1SG.pa\" \":renderPartition.st\" -na\;"));
	f.writeln(string("connectAttr \"Default1.msg\" \":defaultShaderList1.s\" -na\;"));
	for(i=1;i<=snodes.size();i++)
		f.writeln(string("connectAttr \"",snodes[i],".iog\" \"Default1SG.dsm\" -na\;"));
	
}
writeShapeNode: f, myName, pc
{
	string("createNode mesh -n \"",myName,"Shape","\""," -p \"",myName,"\";"); f.writeln(this);
	string("\t","setAttr -k off \".v\"\;"); f.writeln(this);
	string("\t","setAttr \".vir\" yes\;"); f.writeln(this);
	string("\t","setAttr \".vif\" yes\;"); f.writeln(this);
	string("\t","setAttr -s ",pc," \".vt\"\;"); f.writeln(this);
	
	
}
writeTransformNode: f, myName
{
	string("createNode transform -n \"",myName,"\"\;"); f.writeln(this);
}
writeHeader: f, curUnit
{
	string("//Maya ASCII Scene"); f.writeln(this);
	string("//Maya ASCII File generated by Maya2LW[2]"); f.writeln(this);
	string("//Maya2LW[2] Open Source Project by Lernie Ang (faulknermano@yahoo.com)"); f.writeln(this);
	string("//LW-side Exporter tested on Maya 6.5 Platform"); f.writeln(this);
	string("requires maya \"5.0\"\;"); f.writeln(this);
	string("currentUnit -l ",curUnit," -a degree\;"); f.writeln(this);
	
	
	
}

goToMorphMap: oa
{
	pc = editbegin() || error("No points.");
	pnt = points[1];
	
	if(oa.isMapped(pnt))
	{
		myVal = oa.getValue(pnt);
		oa.setValue(pnt,myVal);
	}
	else
	{
		oa.setValue(pnt,nil);
	}
	editend();
}