/*
  PC2_MDD.c
  Source code distributed under the GNU General Purpose License
  Compiled by the BloodShed Dev-C++ Compiler (version 4.9.9.2)
  Author: Lernie Ang
  Date: May 06, 2007
  Description: converts PC2 point cache format by Autodesk into Lightwave MDD format.

  Argument List:
  PC2_MDD.exe (pc2) (mdd)

  History:
  05/06/07 21:00 : version 1.0


*/


#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <math.h>
#include <string.h>

#define MOTOROLA 0
#define INTEL 1
#define SIZEFLOAT 4
#define SIZEINT 4

int readInt (FILE *fp, int pcOrder)
{
   int byte[ 4 ];
  	int n = 3;
	int i;
	int first;
	int stop;
	int step;
	int myTemp;

   if ( pcOrder )
   {
      first = n;
      stop  = -1;
      step  = -1;
   }
   else
   {
      first = 0;
      stop  = n+1;
      step  = 1;
   }

   for ( i = first; i != stop; i += step )
   {
      //byte[ i ] = inFile.readByte();
      byte[i] = fgetc(fp);
   }

   if ( byte[ 0 ] < 128 )
   {
   	return byte[ 0 ] * 16777216
           + byte[ 1 ] * 65536
           + byte[ 2 ] * 256
           + byte[ 3 ];
   }
   else
   {
      return -(( 255 - byte[ 0 ] ) * 16777216
             + ( 255 - byte[ 1 ] ) * 65536
             + ( 255 - byte[ 2 ] ) * 256
             + ( 256 - byte[ 3 ] ));
   }

}
float readFloat (FILE *fp, int pcOrder)
{
	int byte[ 4 ];
  	int n = 3;
	int i;
	int first;
	int stop;
	int step;
	float x;
	float m;
	float e;

   if ( pcOrder )
   {
      first = n;
      stop  = -1;
      step  = -1;
   }
   else
   {
      first = 0;
      stop  = n+1;
      step  = 1;
   }

   for ( i = first; i != stop; i += step )
   {
      //byte[ i ] = inFile.readByte();
      byte[i] = fgetc(fp);
   }

	if ( byte[ 0 ] == 0 && byte[ 1 ] == 0 && byte[ 2 ] == 0 && byte[ 3 ] == 0 )
	 return 0;

	m = 1.0 + (( byte[ 1 ] % 128 ) * 65536 + byte[ 2 ] * 256 + byte[ 3 ] )
	  / 8388608.0;
	e = ( byte[ 0 ] % 128 ) * 2 + ((int)( byte[ 1 ] / 128 )) - 127;
	x = m * pow( 2.0, e );
	if ( byte[ 0 ] > 127 ) x = -x;

	return x;

}

int findDuration (FILE *fp)
{
	int readIntNum = 0;
	int *ptrReadInt;
	ptrReadInt = &readIntNum;

	fseek(fp,28,SEEK_SET);
	fread(ptrReadInt,sizeof(int),1,fp);
	return(*ptrReadInt);
}
int findNumPoints (FILE *fp)
{
	int readIntNum = 0;
	int *ptrReadInt;
	ptrReadInt = &readIntNum;
	fseek(fp,16,SEEK_SET);
	fread(ptrReadInt,4,1,fp);

	return(*ptrReadInt);
}

float findStartFrame (FILE *fp)
{
	float readFloatNum = 0;
	float *ptrReadFloat;
	ptrReadFloat = &readFloatNum;
	fseek(fp,20,SEEK_SET);
	fread(ptrReadFloat,4,1,fp);
	return(*ptrReadFloat);
}
float findSampleRate (FILE *fp)
{
	float readFloatNum;
	float *ptrReadFloat;
	int mysize;
   ptrReadFloat = &readFloatNum;
	//readFloatNum = readFloat(fp,MOTOROLA);
	
	fseek(fp,24,SEEK_SET); 	
   fread(ptrReadFloat,sizeof(float),1,fp);
   //printf("%f - %i \n",*ptrReadFloat,mysize);
	return(*ptrReadFloat);
}


void writeBytes (FILE *fpOut, int byte[], int n, int pcOrder)
{
	int i;
	int first;
	int stop;
	int step;

   if ( pcOrder ) {
      first = n;
      stop  = 0;	
      step  = -1;
   }
   else {
      first = 1;
      stop  = n + 1;
      step  = 1;
   }


   for ( i = first; i != stop; i += step )
   {
   	fputc(byte[i],fpOut);
//    outFile.writeByte( byte[ i ] );

   }

}

void writeInt (FILE *fpOut, int i, int pcOrder)
{
	int j;
   int byte[ 4 ];

   if ( i >= 0 )
   {
      byte[ 1 ] = ( i / 16777216 );
      byte[ 2 ] = ( i / 65536 ) % 256;
      byte[ 3 ] = ( i / 256 ) % 256;
      byte[ 4 ] = i % 256;
   }
   else
   {
      j = -i - 1;
      byte[ 1 ] = 255 - ( j / 16777216 );
      byte[ 2 ] = 255 - ( ( j / 65536 ) % 256 );
      byte[ 3 ] = 255 - ( ( j / 256 ) % 256 );
      byte[ 4 ] = 255 - ( j % 256 );
   }

   writeBytes(fpOut, byte, 4, pcOrder );
}


void writeFloat (FILE *fpOut, double x, int pcOrder)
{
	int e;
	int m;
	double lgx;
   int byte[ 4 ];

   if(x==0)
   {
      byte[ 4 ]=byte[ 3 ]=byte[ 2 ]=byte[ 1 ]=0;
   }
   else
   {
	   lgx = log( fabs( x )) / log( 2.0 );
	   e = (int) ( floor( lgx ));
	   m = (int) ( floor(( pow( 2.0, lgx - e ) - 1.0 ) * 8388608 ));
	   e += 127;

	   byte[ 1 ] = ( e / 2 ) + ( x < 0 ? 128 : 0 );
	   byte[ 2 ] = ( m / 65536 ) + 128 * ( e % 2 );
	   byte[ 3 ] = ( m / 256 ) % 256;
	   byte[ 4 ] = m % 256;
   }
   writeBytes(fpOut, byte, 4, pcOrder );

}

float writeDuration (FILE *fpOut, int duration)
{
	writeInt(fpOut,duration,MOTOROLA);
}
void writeTimeIndices (FILE *fpA, FILE *fpOut, int duration, float startFrame, float samples, float myFPS)
{
	int i;
	int myOffset;
	float readFloatNum = 0;
	float *ptrReadFloat;
	ptrReadFloat = &readFloatNum;
	
	for(i=0;i<duration;i++)
	{

		readFloatNum = (startFrame + (samples * (float) i)) * (1/myFPS);
//		myOffset = (8 + (sizeof(float) * i));
//		fseek(fpA,myOffset,SEEK_END);
		writeFloat(fpOut,*ptrReadFloat,MOTOROLA);
		//printf("%d | %d \n",samples,(startFrame + (samples * (float) i)));
	}

}

float writeNumPoints (FILE *fpOut, int numPoints)
{
	fseek(fpOut,sizeof(int),SEEK_SET);
	writeInt(fpOut,numPoints,MOTOROLA);
}
void writeMDD (FILE *fpA, FILE *fpOut, int duration, int numPoints)
{
	int h;
   int i;
//	int seekVar;
	float readFloatNum = 0;
	float *ptrReadFloat;
	ptrReadFloat = &readFloatNum;


	fseek(fpA,32,SEEK_SET);
//	rewind(fpA);
	for(h=0,i=1;h<duration*numPoints*3;h++,i++)
	{
      if(i > 3)
         i = 1;
		//seekVar = (h*sizeof(float)) + 32;

		//fseek(fpA,seekVar,SEEK_SET);
     	fread(ptrReadFloat,sizeof(float),1,fpA);
     	if(i == 3) // z axis
     	   readFloatNum = -readFloatNum;
		//printf("\ni: %i  j: %i seekVar: %i data: %f",i,j,seekVar,readFloatNum);
		writeFloat(fpOut,*ptrReadFloat,MOTOROLA);
	}

}
void writeVertexPositions (FILE *fpA, FILE *fpOut, int numPoints, int duration)
{
   // copy writeMDD func for x axis compensation
	int i;
   int h;
//	int seekVar;
	float readFloatNum = 0;
	float *ptrReadFloat;
	ptrReadFloat = &readFloatNum;
	fseek(fpA,32,SEEK_SET);
	for(h=0,i=1;h<numPoints*3;h++,i++)
	{
      if(i > 3)
         i = 1;

     	fread(ptrReadFloat,sizeof(float),1,fpA);
     	if(i == 3) // z axis
     	   readFloatNum = -readFloatNum;
		//printf("\ni: %i  j: %i seekVar: %i data: %f",i,j,seekVar,readFloatNum);
		writeFloat(fpOut,*ptrReadFloat,MOTOROLA);
	}
}
int main(int argc, char *argv[]) // argc is number of arguments and argv are the actual arguments in string form
{
	/*
	argument list:
		exe [m2md] [mdd]
	*/

	FILE *fpA, *fpOut;
	int i, j, sourceDuration, sourceNumPoints, mode, lastOffset;
	int readNum = 0, newDuration = 0, timePosition = 0;
	int *ptrReadNum, *ptrSourceDuration, *ptrSourceNumPoints;
	char *char_myFPS;
	double readFloatNum = 0;
	double *ptrReadFloat;

	float timeindex, myFloat, sourceStartTime, sourceEndTime, sourceStartFrame, sourceSample;\
   float myFPS;

	// init pointers
	ptrReadNum = &readNum; // ptrReadNum integer container for line read from file
	ptrReadFloat = &readFloatNum; // ptrReadFloat float container for line read from file

	if(argc < 2) // too few arguments
	{
		fprintf(stderr, "\n Wrong number of arguments" );
		exit(1);
	}

	// ================= CHECK FILES BEGIN =================
	if (( fpA = fopen( argv[1], "rb" )) == NULL )
	{
		fprintf( stderr, "Error opening source file for reading, %s!", argv[1] );
		exit(1);

	}

	if (( fpOut = fopen( argv[2], "wb" )) == NULL )
	{
		fprintf( stderr, "Error opening file for writing, %s!", argv[2] );
		exit(1);


	}

//	strcpy(
	
	myFPS = atof(argv[3]);
	// ================= CHECK FILE END =================

	sourceDuration = findDuration(fpA); // number of samples

	sourceNumPoints = findNumPoints(fpA);

	sourceStartFrame = findStartFrame(fpA);

	sourceSample = findSampleRate(fpA);

   //printf("duration: %i numPoints: %i startFrame: %f sample: %f",sourceDuration,sourceNumPoints,sourceStartFrame,sourceSample,myFPS);


	// write header first
	// source duration
		
	writeDuration(fpOut,sourceDuration);

	// num points
	writeNumPoints(fpOut,sourceNumPoints);
	// list of times

	writeTimeIndices(fpA,fpOut,sourceDuration,sourceStartFrame,sourceSample,myFPS);

	writeVertexPositions(fpA,fpOut,sourceNumPoints,sourceDuration);
	// continue with m2md till lastOffset


	writeMDD(fpA,fpOut,sourceDuration,sourceNumPoints);
	

  	exit(1);
   fclose(fpA);
   fclose(fpOut);

	return 0;
}
