/*
  M2PX_PFX.exe
  Source code distributed under the GNU General Purpose License
  Compiled by the BloodShed Dev-C++ Compiler (version 4.9.7.0)
  Author: Lernie Ang
  Date: 26/03/05
  Description: for use with Maya's MEL to convert series of Maya2LW proprietary M2PX
  					files into one PFX File to be used with LightWave's Particle FX system.
*/

#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <math.h>

#define MOTOROLA 0
#define INTEL 1

void writeBytes (FILE *fpOut, int byte[], int n, int pcOrder)
{
	int i;
	int first;
	int stop;
	int step;
	
   if ( pcOrder ) {
      first = n;
      stop  = 0;
      step  = -1;
   }
   else {
      first = 1;
      stop  = n + 1;
      step  = 1;
   }


   for ( i = first; i != stop; i += step )
   {
   	fputc(byte[i],fpOut);
//    outFile.writeByte( byte[ i ] );
      
   }

}

void writeInt (FILE *fpOut, int i, int pcOrder)
{
	int j;
   int byte[ 4 ];

   if ( i >= 0 )
   {
      byte[ 1 ] = ( i / 16777216 );
      byte[ 2 ] = ( i / 65536 ) % 256;
      byte[ 3 ] = ( i / 256 ) % 256;
      byte[ 4 ] = i % 256;
   }
   else
   {
      j = -i - 1;
      byte[ 1 ] = 255 - ( j / 16777216 );
      byte[ 2 ] = 255 - ( ( j / 65536 ) % 256 );
      byte[ 3 ] = 255 - ( ( j / 256 ) % 256 );
      byte[ 4 ] = 255 - ( j % 256 );
   }

   writeBytes(fpOut, byte, 4, pcOrder );
}


void writeFloat (FILE *fpOut, double x, int pcOrder)
{
	int e;
	int m;
	double lgx;
   int byte[ 4 ];

	double test;

   if(x==0)
   {
      byte[ 4 ]=byte[ 3 ]=byte[ 2 ]=byte[ 1 ]=0;
   }
   else
   {
	   lgx = log( fabs( x )) / log( 2.0 );
	   e = ( floor( lgx ));
	   m = ( floor(( pow( 2.0, lgx - e ) - 1.0 ) * 8388608 ));
	   e += 127;

	   byte[ 1 ] = ( e / 2 ) + ( x < 0 ? 128 : 0 );
	   byte[ 2 ] = ( m / 65536 ) + 128 * ( e % 2 );
	   byte[ 3 ] = ( m / 256 ) % 256;
	   byte[ 4 ] = m % 256;
   }
   writeBytes(fpOut, byte, 4, pcOrder );

}


int main(int argc, char *argv[])
{
	
	FILE *fp, *fpOut;
	int i; // iterator
	int pfxFormatVersion = 3, readNum = 0;
	int *ptr_pfxFormatVersion, *ptrReadNum;
	
	double readFloatNum = 0;
	double *ptrReadFloat;

	char typeHeader;
	char typeParticles;
	char typeAppend;
	char typeNew;


	// init pointers
	ptr_pfxFormatVersion = &pfxFormatVersion;
	ptrReadNum = &readNum;
	
	ptrReadFloat = &readFloatNum;

	typeHeader = 'h';
	typeParticles = 'p';
	typeAppend = 'a';
	typeNew = 'n';
	
	
	
	if(argc < 5) // too few arguments
	{
		fprintf(stderr, "\n Input M2PX File -> Output PFX File -> Header or Particle Mode -> Create Mode" );
		exit(1);
	}

	if (( fp = fopen( argv[1], "rb" )) == NULL )
	{
		fprintf( stderr, "Error opening file for reading, %s!", argv[1] );
		exit(1);
		
	}

	if(*argv[4] == typeAppend)
	{
		if (( fpOut = fopen( argv[2], "ab" )) == NULL )
		{
			fprintf( stderr, "Error opening file for appending, %s!", argv[2] );
			exit(1);
		
		}
	}
	if(*argv[4] == typeNew)
	{
		if (( fpOut = fopen( argv[2], "wb" )) == NULL )
		{
			fprintf( stderr, "Error opening file for writing, %s!", argv[2] );
			exit(1);

		}
	}

	if(*argv[3] == typeHeader) // process pfx header
	{
		fread(ptrReadNum,sizeof(int),1,fp);
		writeInt(fpOut,*ptrReadNum,MOTOROLA);
		fread(ptrReadNum,sizeof(int),1,fp);
		writeInt(fpOut,*ptrReadNum,MOTOROLA);
		fread(ptrReadNum,sizeof(int),1,fp);
		writeInt(fpOut,*ptrReadNum,MOTOROLA);
		
	}
	if(*argv[3] == typeParticles) // process rest of particles
	{
		if(fseek(fp,sizeof(int),SEEK_SET) != 0) // go to start frame
		{
			fprintf(stderr, "\nError using fseek().");
			exit(1);
		}
		
		fread(ptrReadNum,sizeof(int),1,fp); // start frame
		writeInt(fpOut,*ptrReadNum,MOTOROLA);

		if(fseek(fp,-28,SEEK_END) != 0) // go to end frame
		{
			fprintf(stderr, "\nError using fseek().");
			exit(1);
		}
		fread(ptrReadNum,sizeof(int),1,fp); // end frame
		writeInt(fpOut,*ptrReadNum,MOTOROLA);


		rewind(fp); // go back to lifespan

		fread(ptrReadNum,sizeof(int),1,fp); // lifespan
		writeInt(fpOut,*ptrReadNum,MOTOROLA);

		if(fseek(fp,sizeof(int)+sizeof(int),SEEK_SET) != 0) // back to start with int offset
		{
			fprintf(stderr, "\nError using fseek().");
			exit(1);
		}
		
		writeInt(fpOut,1,MOTOROLA);	// particle weight
		writeInt(fpOut,0,MOTOROLA);	// particle Size
		writeInt(fpOut,0,MOTOROLA);	// Particle Resistance

  		while(!feof(fp))
  		{
			for(i=1;i<=3;i++)
   		{
				if(feof(fp))
      			break;

         	fread(ptrReadFloat,sizeof(double),1,fp);
				writeFloat(fpOut,readFloatNum,MOTOROLA);

			}
			fread(ptrReadNum,sizeof(int),1,fp); // lifespan
			fread(ptrReadNum,sizeof(int),1,fp); // framenum
		}
	}
   fclose(fp);
   fclose(fpOut);

	return 0;
}

